@@ -3,6 +3,7 @@ Wild Card Match.
 
 A custom implementation of `glob`.
 """
+from __future__ import annotations
 import os
 import sys
 import re
@@ -12,18 +13,15 @@ import bracex
 from . import _wcparse
 from . import _wcmatch
 from . import util
-from typing import (
-    Optional, Iterator, Iterable, List, AnyStr, Union, Generic,
-    Tuple, Pattern, Callable, Any, Set, Sequence, cast
-)
+from typing import Iterator, Iterable, AnyStr, Generic, Pattern, Callable, Any, Sequence
 
 __all__ = (
     "CASE", "IGNORECASE", "RAWCHARS", "DOTGLOB", "DOTMATCH",
     "EXTGLOB", "EXTMATCH", "GLOBSTAR", "NEGATE", "MINUSNEGATE", "BRACE", "NOUNIQUE",
     "REALPATH", "FOLLOW", "MATCHBASE", "MARK", "NEGATEALL", "NODIR", "FORCEWIN", "FORCEUNIX", "GLOBTILDE",
-    "NODOTDIR", "SCANDOTDIR", "SUPPORT_DIR_FD",
-    "C", "I", "R", "D", "E", "G", "N", "M", "B", "P", "L", "S", "X", 'K', "O", "A", "W", "U", "T", "Q", "Z", "SD",
-    "iglob", "glob", "globmatch", "globfilter", "escape", "raw_escape", "is_magic"
+    "NODOTDIR", "SCANDOTDIR", "SUPPORT_DIR_FD", "GLOBSTARLONG",
+    "C", "I", "R", "D", "E", "G", "N", "M", "B", "P", "L", "S", "X", 'K', "O", "A", "W", "U", "T", "Q", "Z", "SD", "GL",
+    "iglob", "glob", "globmatch", "globfilter", "escape", "is_magic"
 )
 
 # We don't use `util.platform` only because we mock it in tests,
@@ -52,6 +50,7 @@ U = FORCEUNIX = _wcparse.FORCEUNIX
 T = GLOBTILDE = _wcparse.GLOBTILDE
 Q = NOUNIQUE = _wcparse.NOUNIQUE
 Z = NODOTDIR = _wcparse.NODOTDIR
+GL = GLOBSTARLONG = _wcparse.GLOBSTARLONG
 
 K = MARK = 0x1000000
 SD = SCANDOTDIR = 0x2000000
@@ -60,7 +59,6 @@ _PATHLIB = 0x8000000
 
 # Internal flags
 _EXTMATCHBASE = _wcparse._EXTMATCHBASE
-_RTL = _wcparse._RTL
 _NOABSOLUTE = _wcparse._NOABSOLUTE
 _PATHNAME = _wcparse.PATHNAME
 
@@ -71,6 +69,7 @@ FLAG_MASK = (
     DOTMATCH |
     EXTMATCH |
     GLOBSTAR |
+    GLOBSTARLONG |
     NEGATE |
     MINUSNEGATE |
     BRACE |
@@ -86,19 +85,18 @@ FLAG_MASK = (
     NOUNIQUE |
     NODOTDIR |
     _EXTMATCHBASE |
-    _RTL |
     _NOABSOLUTE
 )
 
 _RE_PATHLIB_DOT_NORM = (
     re.compile(r'(?:((?<=^)|(?<=/))\.(?:/|$))+'),
     re.compile(br'(?:((?<=^)|(?<=/))\.(?:/|$))+')
-)  # type: Tuple[Pattern[str], Pattern[bytes]]
+)  # type: tuple[Pattern[str], Pattern[bytes]]
 
 _RE_WIN_PATHLIB_DOT_NORM = (
     re.compile(r'(?:((?<=^)|(?<=[\\/]))\.(?:[\\/]|$))+'),
     re.compile(br'(?:((?<=^)|(?<=[\\/]))\.(?:[\\/]|$))+')
-)  # type: Tuple[Pattern[str], Pattern[bytes]]
+)  # type: tuple[Pattern[str], Pattern[bytes]]
 
 
 def _flag_transform(flags: int) -> int:
@@ -123,7 +121,7 @@ def _flag_transform(flags: int) -> int:
 
 
 class _GlobPart(
-    namedtuple('_GlobPart', ['pattern', 'is_magic', 'is_globstar', 'dir_only', 'is_drive']),
+    namedtuple('_GlobPart', ['pattern', 'is_magic', 'is_globstar', 'is_globstarlong', 'dir_only', 'is_drive']),
 ):
     """File Glob."""
 
@@ -139,6 +137,7 @@ class _GlobSplit(Generic[AnyStr]):
     is "magic", etc.: `["pattern", is_magic, is_globstar, dir_only, is_drive]`.
 
     Example:
+    -------
         `"**/this/is_literal/*magic?/@(magic|part)"`
 
         Would  become:
@@ -162,7 +161,9 @@ class _GlobSplit(Generic[AnyStr]):
         self.unix = _wcparse.is_unix_style(flags)
         self.flags = flags
         self.no_abs = bool(flags & _wcparse._NOABSOLUTE)
-        self.globstar = bool(flags & GLOBSTAR)
+        self.globstarlong = bool(flags & GLOBSTARLONG)
+        self.globstar = self.globstarlong or bool(flags & GLOBSTAR)
+        self.follow = bool(flags & FOLLOW)
         self.matchbase = bool(flags & MATCHBASE)
         self.extmatchbase = bool(flags & _wcparse._EXTMATCHBASE)
         self.tilde = bool(flags & GLOBTILDE)
@@ -185,7 +186,7 @@ class _GlobSplit(Generic[AnyStr]):
             self.sep = '/'
         # Once split, Windows file names will never have `\\` in them,
         # so we can use the Unix magic detect
-        self.magic_symbols = _wcparse._get_magic_symbols(pattern, self.unix, self.flags)[0]  # type: Set[AnyStr]
+        self.magic_symbols = _wcparse._get_magic_symbols(pattern, self.unix, self.flags)[0]  # type: set[AnyStr]
 
     def is_magic(self, name: AnyStr) -> bool:
         """Check if name contains magic characters."""
@@ -209,8 +210,8 @@ class _GlobSplit(Generic[AnyStr]):
                 # Handle escapes
                 try:
                     self._references(i, True)
-                except _wcparse.PathNameException:
-                    raise StopIteration
+                except _wcparse.PathNameException as e:
+                    raise StopIteration from e
             elif c == '/':
                 raise StopIteration
             c = next(i)
@@ -272,24 +273,25 @@ class _GlobSplit(Generic[AnyStr]):
 
         return success
 
-    def store(self, value: AnyStr, l: List[_GlobPart], dir_only: bool) -> None:
+    def store(self, value: AnyStr, l: list[_GlobPart], dir_only: bool) -> None:
         """Group patterns by literals and potential magic patterns."""
 
         if l and value in (b'', ''):
             return
 
-        globstar = value in (b'**', '**') and self.globstar
+        globstarlong = self.globstarlong and value in (b'***', '***')
+        globstar = globstarlong or (self.globstar and value in (b'**', '**'))
         magic = self.is_magic(value)
         if magic:
-            v = cast(Pattern[AnyStr], _wcparse._compile(value, self.flags))  # type: Union[Pattern[AnyStr], AnyStr]
+            v = _wcparse._compile(value, self.flags)  # type: Pattern[AnyStr] | AnyStr
         else:
             v = value
         if globstar and l and l[-1].is_globstar:
-            l[-1] = _GlobPart(v, magic, globstar, dir_only, False)
+            l[-1] = _GlobPart(v, magic, globstar, globstarlong, dir_only, False)
         else:
-            l.append(_GlobPart(v, magic, globstar, dir_only, False))
+            l.append(_GlobPart(v, magic, globstar, globstarlong, dir_only, False))
 
-    def split(self) -> List[_GlobPart]:
+    def split(self) -> list[_GlobPart]:
         """Start parsing the pattern."""
 
         split_index = []
@@ -309,15 +311,19 @@ class _GlobSplit(Generic[AnyStr]):
         if self.win_drive_detect:
             root_specified, drive, slash, end = _wcparse._get_win_drive(pattern)
             if drive is not None:
-                parts.append(_GlobPart(drive.encode('latin-1') if is_bytes else drive, False, False, True, True))
+                parts.append(_GlobPart(drive.encode('latin-1') if is_bytes else drive, False, False, False, True, True))
                 start = end - 1
                 i.advance(start)
             elif drive is None and root_specified:
-                parts.append(_GlobPart(b'\\' if is_bytes else '\\', False, False, True, True))
-                start = 1
-                i.advance(2)
+                parts.append(_GlobPart(b'\\' if is_bytes else '\\', False, False, False, True, True))
+                if pattern.startswith('/'):
+                    start = 0
+                    i.advance(1)
+                else:
+                    start = 1
+                    i.advance(2)
         elif not self.win_drive_detect and pattern.startswith('/'):
-            parts.append(_GlobPart(b'/' if is_bytes else '/', False, False, True, True))
+            parts.append(_GlobPart(b'/' if is_bytes else '/', False, False, False, True, True))
             start = 0
             i.advance(1)
 
@@ -345,23 +351,30 @@ class _GlobSplit(Generic[AnyStr]):
 
         for split, offset in split_index:
             value = pattern[start + 1:split]
-            self.store(cast(AnyStr, value.encode('latin-1') if is_bytes else value), parts, True)
+            self.store(value.encode('latin-1') if is_bytes else value, parts, True)  # type: ignore[arg-type]
             start = split + offset
 
         if start < len(pattern):
             value = pattern[start + 1:]
             if value:
-                self.store(cast(AnyStr, value.encode('latin-1') if is_bytes else value), parts, False)
+                self.store(value.encode('latin-1') if is_bytes else value, parts, False)  # type: ignore[arg-type]
 
         if len(pattern) == 0:
-            parts.append(_GlobPart(pattern.encode('latin-1') if is_bytes else pattern, False, False, False, False))
+            parts.append(
+                _GlobPart(pattern.encode('latin-1') if is_bytes else pattern, False, False, False, False, False)
+            )
 
         if (
             (self.extmatchbase and not parts[0].is_drive) or
             (self.matchbase and len(parts) == 1 and not parts[0].dir_only)
         ):
-            self.globstar = True
-            parts.insert(0, _GlobPart(b'**' if is_bytes else '**', True, True, True, False))
+            if self.globstarlong and self.follow:
+                gstar = b'***' if is_bytes else '***'  # type: Any
+                is_globstarlong = True
+            else:
+                gstar = b'**' if is_bytes else '**'
+                is_globstarlong = False
+            parts.insert(0, _GlobPart(gstar, True, True, is_globstarlong, True, False))
 
         if self.no_abs and parts and parts[0].is_drive:
             raise ValueError('The pattern must be a relative path pattern')
@@ -374,12 +387,12 @@ class Glob(Generic[AnyStr]):
 
     def __init__(
         self,
-        pattern: Union[AnyStr, Sequence[AnyStr]],
+        pattern: AnyStr | Sequence[AnyStr],
         flags: int = 0,
-        root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-        dir_fd: Optional[int] = None,
+        root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+        dir_fd: int | None = None,
         limit: int = _wcparse.PATTERN_LIMIT,
-        exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+        exclude: AnyStr | Sequence[AnyStr] | None = None
     ) -> None:
         """Initialize the directory walker object."""
 
@@ -389,10 +402,10 @@ class Glob(Generic[AnyStr]):
         if epats is not None:
             flags = _wcparse.no_negate_flags(flags)
 
-        self.pattern = []  # type: List[List[_GlobPart]]
-        self.npatterns = []  # type: List[Pattern[AnyStr]]
-        self.seen = set()  # type: Set[AnyStr]
-        self.dir_fd = dir_fd if SUPPORT_DIR_FD else None  # type: Optional[int]
+        self.pattern = []  # type: list[list[_GlobPart]]
+        self.npatterns = []  # type: list[Pattern[AnyStr]]
+        self.seen = set()  # type: set[AnyStr]
+        self.dir_fd = dir_fd if SUPPORT_DIR_FD else None  # type: int | None
         self.nounique = bool(flags & NOUNIQUE)  # type: bool
         self.mark = bool(flags & MARK)  # type: bool
         # Only scan for `.` and `..` if it is specifically requested.
@@ -408,19 +421,17 @@ class Glob(Generic[AnyStr]):
         self.pathlib = bool(flags & _PATHLIB)  # type: bool
         if self.pathlib:
             flags ^= _PATHLIB
-        # Right to left searching is only for matching
-        if flags & _RTL:  # pragma: no cover
-            flags ^= _RTL
         self.flags = _flag_transform(flags | REALPATH)  # type: int
         self.negate_flags = self.flags | DOTMATCH | _wcparse._NO_GLOBSTAR_CAPTURE  # type: int
         if not self.scandotdir and not self.flags & NODOTDIR:
             self.flags |= NODOTDIR
         self.raw_chars = bool(self.flags & RAWCHARS)  # type: bool
-        self.follow_links = bool(self.flags & FOLLOW)  # type: bool
         self.dot = bool(self.flags & DOTMATCH)  # type: bool
         self.unix = not bool(self.flags & FORCEWIN)  # type: bool
         self.negate = bool(self.flags & NEGATE)  # type: bool
-        self.globstar = bool(self.flags & GLOBSTAR)  # type: bool
+        self.globstarlong = bool(self.flags & GLOBSTARLONG)  # type: bool
+        self.globstar = self.globstarlong or bool(self.flags & GLOBSTAR)  # type: bool
+        self.follow_links = bool(self.flags & FOLLOW) and not self.globstarlong  # type: bool
         self.braces = bool(self.flags & BRACE)  # type: bool
         self.matchbase = bool(self.flags & MATCHBASE)  # type: bool
         self.case_sensitive = _wcparse.get_case(self.flags)  # type: bool
@@ -430,13 +441,13 @@ class Glob(Generic[AnyStr]):
         if isinstance(pats[0], bytes):
             ptype = util.BYTES
             self.current = b'.'  # type: AnyStr
-            self.specials = (b'.', b'..')  # type: Tuple[AnyStr, ...]
+            self.specials = (b'.', b'..')  # type: tuple[AnyStr, ...]
             self.empty = b''  # type: AnyStr
             self.stars = b'**'  # type: AnyStr
             self.sep = b'\\' if forcewin else b'/'  # type: AnyStr
-            self.seps = (b'/', self.sep) if forcewin else (self.sep,)  # type: Tuple[AnyStr, ...]
-            self.re_pathlib_norm = cast(Pattern[AnyStr], _RE_WIN_PATHLIB_DOT_NORM[ptype])  # type: Pattern[AnyStr]
-            self.re_no_dir = cast(Pattern[AnyStr], _wcparse.RE_WIN_NO_DIR[ptype])  # type: Pattern[AnyStr]
+            self.seps = (b'/', self.sep) if forcewin else (self.sep,)  # type: tuple[AnyStr, ...]
+            self.re_pathlib_norm = _RE_WIN_PATHLIB_DOT_NORM[ptype]  # type: Pattern[AnyStr]  # type: ignore[assignment]
+            self.re_no_dir = _wcparse.RE_WIN_NO_DIR[ptype]  # type: Pattern[AnyStr]  # type: ignore[assignment]
         else:
             ptype = util.UNICODE
             self.current = '.'
@@ -445,15 +456,13 @@ class Glob(Generic[AnyStr]):
             self.stars = '**'
             self.sep = '\\' if forcewin else '/'
             self.seps = ('/', self.sep) if forcewin else (self.sep,)
-            self.re_pathlib_norm = cast(Pattern[AnyStr], _RE_WIN_PATHLIB_DOT_NORM[ptype])
-            self.re_no_dir = cast(Pattern[AnyStr], _wcparse.RE_WIN_NO_DIR[ptype])
+            self.re_pathlib_norm = _RE_WIN_PATHLIB_DOT_NORM[ptype]  # type: ignore[assignment]
+            self.re_no_dir = _wcparse.RE_WIN_NO_DIR[ptype]  # type: ignore[assignment]
 
         temp = os.fspath(root_dir) if root_dir is not None else self.current
-        if not isinstance(temp, type(pats[0])):
+        if not isinstance(temp, bytes if ptype else str):
             raise TypeError(
-                'Pattern and root_dir should be of the same type, not {} and {}'.format(
-                    type(pats[0]), type(temp)
-                )
+                f'Pattern and root_dir should be of the same type, not {type(pats[0])} and {type(temp)}'
             )
 
         self.root_dir = temp  # type: AnyStr
@@ -462,7 +471,7 @@ class Glob(Generic[AnyStr]):
         if epats is not None:
             self._parse_patterns(epats, force_negate=True)
 
-    def _iter_patterns(self, patterns: Sequence[AnyStr], force_negate: bool = False) -> Iterator[Tuple[bool, AnyStr]]:
+    def _iter_patterns(self, patterns: Sequence[AnyStr], force_negate: bool = False) -> Iterator[tuple[bool, AnyStr]]:
         """Iterate expanded patterns."""
 
         seen = set()
@@ -471,11 +480,12 @@ class Glob(Generic[AnyStr]):
             for p in patterns:
                 p = util.norm_pattern(p, not self.unix, self.raw_chars)
                 count = 0
-                for count, expanded in enumerate(_wcparse.expand(p, self.flags, self.current_limit), 1):
+                for expanded in _wcparse.expand(p, self.flags, self.current_limit):
+                    count += 1
                     total += 1
                     if 0 < self.limit < total:
                         raise _wcparse.PatternLimitException(
-                            "Pattern limit exceeded the limit of {:d}".format(self.limit)
+                            f"Pattern limit exceeded the limit of {self.limit:d}"
                         )
                     # Filter out duplicate patterns. If `NOUNIQUE` is enabled,
                     # we only want to filter on negative patterns as they are
@@ -491,10 +501,10 @@ class Glob(Generic[AnyStr]):
                     self.current_limit -= count
                     if self.current_limit < 1:
                         self.current_limit = 1
-        except bracex.ExpansionLimitException:
+        except bracex.ExpansionLimitException as e:
             raise _wcparse.PatternLimitException(
-                "Pattern limit exceeded the limit of {:d}".format(self.limit)
-            )
+                f"Pattern limit exceeded the limit of {self.limit:d}"
+            ) from e
 
     def _parse_patterns(self, patterns: Sequence[AnyStr], force_negate: bool = False) -> None:
         """Parse patterns."""
@@ -504,7 +514,7 @@ class Glob(Generic[AnyStr]):
                 # Treat the inverse pattern as a normal pattern if it matches, we will exclude.
                 # This is faster as compiled patterns usually compare the include patterns first,
                 # and then the exclude, but glob will already know it wants to include the file.
-                self.npatterns.append(cast(Pattern[AnyStr], _wcparse._compile(p, self.negate_flags)))
+                self.npatterns.append(_wcparse._compile(p, self.negate_flags))
             else:
                 self.pattern.append(_GlobSplit(p, self.flags).split())
 
@@ -561,16 +571,16 @@ class Glob(Generic[AnyStr]):
 
         return bool(self.npatterns and self._match_excluded(path, is_dir))
 
-    def _match_literal(self, a: AnyStr, b: Optional[AnyStr] = None) -> bool:
+    def _match_literal(self, a: AnyStr, b: AnyStr | None = None) -> bool:
         """Match two names."""
 
         return a.lower() == b if not self.case_sensitive else a == b
 
-    def _get_matcher(self, target: Optional[Union[AnyStr, Pattern[AnyStr]]]) -> Optional[Callable[..., Any]]:
+    def _get_matcher(self, target: AnyStr | Pattern[AnyStr] | None) -> Callable[..., Any] | None:
         """Get deep match."""
 
         if target is None:
-            matcher = None  # type: Optional[Callable[..., Any]]
+            matcher = None  # type: Callable[..., Any] | None
         elif isinstance(target, (str, bytes)):
             # Plain text match
             if not self.case_sensitive:
@@ -603,13 +613,13 @@ class Glob(Generic[AnyStr]):
         else:
             return os.path.join(self.root_dir, path)
 
-    def _iter(self, curdir: Optional[AnyStr], dir_only: bool, deep: bool) -> Iterator[Tuple[AnyStr, bool, bool, bool]]:
+    def _iter(self, curdir: AnyStr | None, dir_only: bool, deep: bool) -> Iterator[tuple[AnyStr, bool, bool, bool]]:
         """Iterate the directory."""
 
         try:
-            fd = None  # type: Optional[int]
+            fd = None  # type: int | None
             if self.is_abs_pattern and curdir:
-                scandir = curdir  # type: Union[AnyStr, int]
+                scandir = curdir  # type: AnyStr | int
             elif self.dir_fd is not None:
                 fd = scandir = os.open(
                     os.path.join(self.root_dir, curdir) if curdir else self.root_dir,
@@ -636,7 +646,7 @@ class Glob(Generic[AnyStr]):
                                 is_link = False
                             if (not dir_only or is_dir):
                                 yield f.name, is_dir, hidden, is_link  # type: ignore[misc]
-                        except OSError:  # pragma: no cover
+                        except OSError:  # pragma: no cover # noqa: PERF203
                             pass
             finally:
                 if fd is not None:
@@ -648,10 +658,11 @@ class Glob(Generic[AnyStr]):
     def _glob_dir(
         self,
         curdir: AnyStr,
-        matcher: Optional[Callable[..., Any]],
+        matcher: Callable[..., Any] | None,
         dir_only: bool = False,
-        deep: bool = False
-    ) -> Iterator[Tuple[AnyStr, bool]]:
+        deep: bool = False,
+        globstar_follow: bool = False
+    ) -> Iterator[tuple[AnyStr, bool]]:
         """Recursive directory glob."""
 
         files = list(self._iter(curdir, dir_only, deep))
@@ -662,14 +673,14 @@ class Glob(Generic[AnyStr]):
                 continue
 
             path = os.path.join(curdir, file)
-            follow = not is_link or self.follow_links
-            if (matcher is None and not hidden and (follow or not deep)) or (matcher and matcher(file)):
+            if (matcher is None and not hidden) or (matcher and matcher(file)):
                 yield path, is_dir
 
+            follow = not is_link or self.follow_links or globstar_follow
             if deep and not hidden and is_dir and follow:
-                yield from self._glob_dir(path, matcher, dir_only, deep)
+                yield from self._glob_dir(path, matcher, dir_only, deep, globstar_follow)
 
-    def _glob(self, curdir: AnyStr, part: _GlobPart, rest: List[_GlobPart]) -> Iterator[Tuple[AnyStr, bool]]:
+    def _glob(self, curdir: AnyStr, part: _GlobPart, rest: list[_GlobPart]) -> Iterator[tuple[AnyStr, bool]]:
         """
         Handle glob flow.
 
@@ -687,6 +698,7 @@ class Glob(Generic[AnyStr]):
         dir_only = part.dir_only
         target = part.pattern
         is_globstar = part.is_globstar
+        is_globstarlong = part.is_globstarlong
 
         if is_magic and is_globstar:
             # Glob star directory `**`.
@@ -722,7 +734,7 @@ class Glob(Generic[AnyStr]):
                 yield os.path.join(curdir, self.empty), True
 
             # Search
-            for path, is_dir in self._glob_dir(curdir, matcher, dir_only, deep=True):
+            for path, is_dir in self._glob_dir(curdir, matcher, dir_only, deep=True, globstar_follow=is_globstarlong):
                 if this:
                     yield from self._glob(path, this, rest[:])
                 else:
@@ -744,7 +756,7 @@ class Glob(Generic[AnyStr]):
                 else:
                     yield path, is_dir
 
-    def _get_starting_paths(self, curdir: AnyStr, dir_only: bool) -> List[Tuple[AnyStr, bool]]:
+    def _get_starting_paths(self, curdir: AnyStr, dir_only: bool) -> list[tuple[AnyStr, bool]]:
         """
         Get the starting location.
 
@@ -758,7 +770,7 @@ class Glob(Generic[AnyStr]):
             results = []
             matcher = self._get_matcher(curdir)
             files = list(self._iter(None, dir_only, False))
-            for file, is_dir, hidden, is_link in files:
+            for file, is_dir, _hidden, _is_link in files:
                 if file not in self.specials and (matcher is None or matcher(file)):
                     results.append((file, is_dir))
         else:
@@ -793,9 +805,9 @@ class Glob(Generic[AnyStr]):
     def glob(self) -> Iterator[AnyStr]:
         """Starts off the glob iterator."""
 
-        curdir = self.current
-
         for pattern in self.pattern:
+            curdir = self.current
+
             # If the pattern ends with `/` we return the files ending with `/`.
             dir_only = pattern[-1].dir_only if pattern else False
             self.is_abs_pattern = pattern[0].is_drive if pattern else False
@@ -844,13 +856,13 @@ class Glob(Generic[AnyStr]):
 
 
 def iglob(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+    exclude: AnyStr | Sequence[AnyStr] | None = None
 ) -> Iterator[AnyStr]:
     """Glob."""
 
@@ -861,26 +873,26 @@ def iglob(
 
 
 def glob(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> List[AnyStr]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> list[AnyStr]:
     """Glob."""
 
     return list(iglob(patterns, flags=flags, root_dir=root_dir, dir_fd=dir_fd, limit=limit, exclude=exclude))
 
 
 def translate(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> Tuple[List[AnyStr], List[AnyStr]]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> tuple[list[AnyStr], list[AnyStr]]:
     """Translate glob pattern."""
 
     flags = _flag_transform(flags)
@@ -888,14 +900,14 @@ def translate(
 
 
 def globmatch(
-    filename: Union[AnyStr, 'os.PathLike[AnyStr]'],
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    filename: AnyStr | os.PathLike[AnyStr],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+    exclude: AnyStr | Sequence[AnyStr] | None = None
 ) -> bool:
     """
     Check if filename matches pattern.
@@ -916,15 +928,15 @@ def globmatch(
 
 
 def globfilter(
-    filenames: Iterable[Union[AnyStr, 'os.PathLike[AnyStr]']],
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    filenames: Iterable[AnyStr | os.PathLike[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> List[Union[AnyStr, 'os.PathLike[AnyStr]']]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> list[AnyStr | os.PathLike[AnyStr]]:
     """Filter names using pattern."""
 
     # Shortcut out if we have no patterns
@@ -933,7 +945,7 @@ def globfilter(
 
     rdir = os.fspath(root_dir) if root_dir is not None else root_dir
 
-    matches = []  # type: List[Union[AnyStr, 'os.PathLike[AnyStr]']]
+    matches = []  # type: list[AnyStr | os.PathLike[AnyStr]]
     flags = _flag_transform(flags)
     obj = _wcparse.compile(patterns, flags, limit, exclude)
 
@@ -944,16 +956,7 @@ def globfilter(
     return matches
 
 
-@util.deprecated("This function will be removed in 9.0.")
-def raw_escape(pattern: AnyStr, unix: Optional[bool] = None, raw_chars: bool = True) -> AnyStr:
-    """Apply raw character transform before applying escape."""
-
-    return _wcparse.escape(
-        util.norm_pattern(pattern, False, raw_chars, True), unix=unix, pathname=True, raw=True
-    )
-
-
-def escape(pattern: AnyStr, unix: Optional[bool] = None) -> AnyStr:
+def escape(pattern: AnyStr, unix: bool | None = None) -> AnyStr:
     """Escape."""
 
     return _wcparse.escape(pattern, unix=unix)
